>From a0a21dc691e580302f4109cfe3a9b9dd7f7b39ce Mon Sep 17 00:00:00 2001
From: Craig Ringer <craig@2ndquadrant.com>
Date: Thu, 19 Jun 2014 13:56:57 +0800
Subject: [PATCH] Fix use of server-queried version before connection

psqlODBC attempted to use the version that's queried from the server before it
was obtained from the server during the connection setup. Because the server
version is initialized to 0.0, tests for "server version >= 7.4" or "server
version >= 6.4" using the PG_VERSION_LT macro were always returning false.

Instead, use PROTOCOL_64 and PROTOCOL_74 tests against conninfo. These don't
offer any "greater than" form, so just test both. There's little point adding
anything prettier when it can all be removed when v1/v2 protocol support is
removed soon anyway.

The main symptom of this issue was that psqlODBC was sending an invalid
protocol message for the v3 protocol, "Q \0", causing the server to wait for
the second half of the length word forever. At connection close, the server
would emit:

    unexpected EOF within message length word

This issue was observed while tracking down problems with XA transactions in
MSDTC failing to commit after a successful prepare because an isolated tx
created via getLockedXAConn() got stuck here, but is not confined to that
problem.
---
 connection.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/connection.c b/connection.c
index 52f5463..c6ff21e 100644
--- a/connection.c
+++ b/connection.c
@@ -2747,7 +2747,7 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 				empty_reqs;
 	BOOL		ReadyToReturn = FALSE,
 				query_completed = FALSE,
-				beforeV2 = PG_VERSION_LT(self, 6.4),
+				beforeV2 = !(PROTOCOL_64(&self->connInfo) || PROTOCOL_74(&self->connInfo)),
 				aborted = FALSE,
 				used_passed_result_object = FALSE,
 			discard_next_begin = FALSE,
-- 
1.9.0

